@@ -46,10 +46,10 @@ AuthTypeList MODVAR AuthTypeLists[] = {
 };
 
 /* Helper function for Auth_AutoDetectHashType() */
-static int parsepass(char *str, char **salt, char **hash)
+static int parsepass(const char *str, char **salt, char **hash)
 {
 	static char saltbuf[512], hashbuf[512];
-	char *p;
+	const char *p;
 	int max;
 
 	/* Syntax: $<salt>$<hash> */
@@ -72,7 +72,7 @@ static int parsepass(char *str, char **salt, char **hash)
 /** Auto detect hash type for input hash 'hash'.
  * Will fallback to AUTHTYPE_PLAINTEXT when not found (or invalid).
  */
-int Auth_AutoDetectHashType(char *hash)
+int Auth_AutoDetectHashType(const char *hash)
 {
 	static char hashbuf[256];
 	char *saltstr, *hashstr;
@@ -80,12 +80,12 @@ int Auth_AutoDetectHashType(char *hash)
 
 	if (!strchr(hash, '$'))
 	{
-		/* SHA256 SSL fingerprint perhaps?
+		/* SHA256 certificate fingerprint perhaps?
 		 * These are exactly 64 bytes (00112233..etc..) or 95 bytes (00:11:22:33:etc) in size.
 		 */
 		if ((strlen(hash) == 64) || (strlen(hash) == 95))
 		{
-			char *p;
+			const char *p;
 			char *hexchars = "0123456789abcdefABCDEF";
 			for (p = hash; *p; p++)
 				if ((*p != ':') && !strchr(hexchars, *p))
@@ -96,7 +96,7 @@ int Auth_AutoDetectHashType(char *hash)
 
 		if (strlen(hash) == 44)
 		{
-			char *p;
+			const char *p;
 			char *b64chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
 			for (p = hash; *p; p++)
 				if (!strchr(b64chars, *p))
@@ -134,7 +134,7 @@ int Auth_AutoDetectHashType(char *hash)
  *               than trying to determine the type on the 'hash' parameter.
  *               Or leave NULL, then we use hash autodetection.
  */
-AuthenticationType Auth_FindType(char *hash, char *type)
+AuthenticationType Auth_FindType(const char *hash, const char *type)
 {
 	if (type)
 	{
@@ -163,25 +163,25 @@ int Auth_CheckError(ConfigEntry *ce)
 	AuthenticationType type = AUTHTYPE_PLAINTEXT;
 	X509 *x509_filecert = NULL;
 	FILE *x509_f = NULL;
-	if (!ce->ce_vardata)
+	if (!ce->value)
 	{
 		config_error("%s:%i: authentication module failure: missing parameter",
-			ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+			ce->file->filename, ce->line_number);
 		return -1;
 	}
-	if (ce->ce_entries && ce->ce_entries->ce_next)
+	if (ce->items && ce->items->next)
 	{
 		config_error("%s:%i: you may not have multiple authentication methods",
-			ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+			ce->file->filename, ce->line_number);
 		return -1;
 	}
 
-	type = Auth_FindType(ce->ce_vardata, ce->ce_entries ? ce->ce_entries->ce_varname : NULL);
+	type = Auth_FindType(ce->value, ce->items ? ce->items->name : NULL);
 	if (type == -1)
 	{
 		config_error("%s:%i: authentication module failure: %s is not an implemented/enabled authentication method",
-			ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
-			ce->ce_entries->ce_varname);
+			ce->file->filename, ce->line_number,
+			ce->items->name);
 		return -1;
 	}
 
@@ -189,19 +189,19 @@ int Auth_CheckError(ConfigEntry *ce)
 	{
 		case AUTHTYPE_UNIXCRYPT:
 			/* If our data is like 1 or none, we just let em through .. */
-			if (strlen(ce->ce_vardata) < 2)
+			if (strlen(ce->value) < 2)
 			{
 				config_error("%s:%i: authentication module failure: AUTHTYPE_UNIXCRYPT: no salt (crypt strings will always be >2 in length)",
-					ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+					ce->file->filename, ce->line_number);
 				return -1;
 			}
 			break;
 		case AUTHTYPE_TLS_CLIENTCERT:
-			convert_to_absolute_path(&ce->ce_vardata, CONFDIR);
-			if (!(x509_f = fopen(ce->ce_vardata, "r")))
+			convert_to_absolute_path(&ce->value, CONFDIR);
+			if (!(x509_f = fopen(ce->value, "r")))
 			{
 				config_error("%s:%i: authentication module failure: AUTHTYPE_TLS_CLIENTCERT: error opening file %s: %s",
-					ce->ce_fileptr->cf_filename, ce->ce_varlinenum, ce->ce_vardata, strerror(errno));
+					ce->file->filename, ce->line_number, ce->value, strerror(errno));
 				return -1;
 			}
 			x509_filecert = PEM_read_X509(x509_f, NULL, NULL, NULL);
@@ -209,7 +209,7 @@ int Auth_CheckError(ConfigEntry *ce)
 			if (!x509_filecert)
 			{
 				config_error("%s:%i: authentication module failure: AUTHTYPE_TLS_CLIENTCERT: PEM_read_X509 errored in file %s (format error?)",
-					ce->ce_fileptr->cf_filename, ce->ce_varlinenum, ce->ce_vardata);
+					ce->file->filename, ce->line_number, ce->value);
 				return -1;
 			}
 			X509_free(x509_filecert);
@@ -226,19 +226,19 @@ int Auth_CheckError(ConfigEntry *ce)
 	 * with normally at least 5000 rounds (unless deliberately weakened
 	 * by the user).
 	 */
-	if ((type == AUTHTYPE_UNIXCRYPT) && strncmp(ce->ce_vardata, "$5", 2) &&
-	    strncmp(ce->ce_vardata, "$6", 2) && !strstr(ce->ce_vardata, "$rounds"))
+	if ((type == AUTHTYPE_UNIXCRYPT) && strncmp(ce->value, "$5", 2) &&
+	    strncmp(ce->value, "$6", 2) && !strstr(ce->value, "$rounds"))
 	{
 		config_warn("%s:%i: Using simple crypt for authentication is not recommended. "
 		            "Consider using the more secure auth-type 'argon2' instead. "
 		            "See https://www.unrealircd.org/docs/Authentication_types for the complete list.",
-                            ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+                            ce->file->filename, ce->line_number);
 		/* do not return, not an error. */
 	}
-	if ((type == AUTHTYPE_PLAINTEXT) && (strlen(ce->ce_vardata) > PASSWDLEN))
+	if ((type == AUTHTYPE_PLAINTEXT) && (strlen(ce->value) > PASSWDLEN))
 	{
 		config_error("%s:%i: passwords length may not exceed %d",
-			ce->ce_fileptr->cf_filename, ce->ce_varlinenum, PASSWDLEN);
+			ce->file->filename, ce->line_number, PASSWDLEN);
 		return -1;
 	}
 	return 1;
@@ -252,12 +252,12 @@ AuthConfig *AuthBlockToAuthConfig(ConfigEntry *ce)
 	AuthenticationType type = AUTHTYPE_PLAINTEXT;
 	AuthConfig *as = NULL;
 
-	type = Auth_FindType(ce->ce_vardata, ce->ce_entries ? ce->ce_entries->ce_varname : NULL);
+	type = Auth_FindType(ce->value, ce->items ? ce->items->name : NULL);
 	if (type == AUTHTYPE_INVALID)
 		type = AUTHTYPE_PLAINTEXT;
 
 	as = safe_alloc(sizeof(AuthConfig));
-	safe_strdup(as->data, ce->ce_vardata);
+	safe_strdup(as->data, ce->value);
 	as->type = type;
 	return as;
 }
@@ -279,7 +279,7 @@ void Auth_FreeAuthConfig(AuthConfig *as)
 #define RAWSALTLEN		6
 #define REALSALTLEN		12
 
-static int authcheck_argon2(Client *client, AuthConfig *as, char *para)
+static int authcheck_argon2(Client *client, AuthConfig *as, const char *para)
 {
 	argon2_type hashtype;
 
@@ -304,7 +304,7 @@ static int authcheck_argon2(Client *client, AuthConfig *as, char *para)
 	return 0; /* NO MATCH or error */
 }
 
-static int authcheck_bcrypt(Client *client, AuthConfig *as, char *para)
+static int authcheck_bcrypt(Client *client, AuthConfig *as, const char *para)
 {
 	char data[512]; /* NOTE: only 64 required by BF_crypt() */
 	char *str;
@@ -324,7 +324,7 @@ static int authcheck_bcrypt(Client *client, AuthConfig *as, char *para)
 	return 0; /* NO MATCH */
 }
 
-static int authcheck_tls_clientcert(Client *client, AuthConfig *as, char *para)
+static int authcheck_tls_clientcert(Client *client, AuthConfig *as, const char *para)
 {
 	X509 *x509_clientcert = NULL;
 	X509 *x509_filecert = NULL;
@@ -358,11 +358,11 @@ static int authcheck_tls_clientcert(Client *client, AuthConfig *as, char *para)
 	return 1;
 }
 
-static int authcheck_tls_clientcert_fingerprint(Client *client, AuthConfig *as, char *para)
+static int authcheck_tls_clientcert_fingerprint(Client *client, AuthConfig *as, const char *para)
 {
 	int i, k;
 	char hexcolon[EVP_MAX_MD_SIZE * 3 + 1];
-	char *fp;
+	const char *fp;
 
 	if (!client->local->ssl)
 		return 0;
@@ -389,12 +389,12 @@ static int authcheck_tls_clientcert_fingerprint(Client *client, AuthConfig *as,
 	return 1;
 }
 
-static int authcheck_spkifp(Client *client, AuthConfig *as, char *para)
+static int authcheck_spkifp(Client *client, AuthConfig *as, const char *para)
 {
-	char *fp = spki_fingerprint(client);
+	const char *fp = spki_fingerprint(client);
 
 	if (!fp)
-		return 0; /* auth failed: not SSL (or other failure) */
+		return 0; /* auth failed: not TLS or some other failure */
 
 	if (strcasecmp(as->data, fp))
 		return 0; /* auth failed: mismatch */
@@ -420,7 +420,7 @@ static int authcheck_spkifp(Client *client, AuthConfig *as, char *para)
  * - The return value was different in versions before UnrealIRCd 5.0.0!
  * - In older versions a NULL 'as' was treated as an allow, now it's deny.
  */
-int Auth_Check(Client *client, AuthConfig *as, char *para)
+int Auth_Check(Client *client, AuthConfig *as, const char *para)
 {
 	extern char *crypt();
 	char *res;
@@ -435,8 +435,9 @@ int Auth_Check(Client *client, AuthConfig *as, char *para)
 				return 0;
 			if (!strcmp(as->data, "changemeplease") && !strcmp(para, as->data))
 			{
-				sendto_realops("Rejecting default password 'changemeplease'. "
-				               "Please change the password in the configuration file.");
+				unreal_log(ULOG_INFO, "auth", "AUTH_REJECT_DEFAULT_PASSWORD", client,
+				           "Rejecting default password 'changemeplease'. "
+				           "Please change the password in the configuration file.");
 				return 0;
 			}
 			/* plain text compare */
@@ -479,7 +480,7 @@ int Auth_Check(Client *client, AuthConfig *as, char *para)
 #define UNREALIRCD_ARGON2_DEFAULT_HASH_LENGTH           32
 #define UNREALIRCD_ARGON2_DEFAULT_SALT_LENGTH           (128/8)
 
-static char *mkpass_argon2(char *para)
+static char *mkpass_argon2(const char *para)
 {
 	static char buf[512];
 	char salt[UNREALIRCD_ARGON2_DEFAULT_SALT_LENGTH];
@@ -511,7 +512,7 @@ static char *mkpass_argon2(char *para)
 	return buf;
 }
 
-static char *mkpass_bcrypt(char *para)
+static char *mkpass_bcrypt(const char *para)
 {
 	static char buf[128];
 	char data[512]; /* NOTE: only 64 required by BF_crypt() */
@@ -547,7 +548,7 @@ static char *mkpass_bcrypt(char *para)
  * @param text  The password in plaintext.
  * @returns The hashed password.
  */
-char *Auth_Hash(AuthenticationType type, char *text)
+const char *Auth_Hash(AuthenticationType type, const char *text)
 {
 	switch (type)
 	{
