@@ -384,7 +384,7 @@ void unreal_delete_match(Match *m)
 	safe_free(m);
 }
 
-Match *unreal_create_match(MatchType type, char *str, char **error)
+Match *unreal_create_match(MatchType type, const char *str, char **error)
 {
 	Match *m = safe_alloc(sizeof(Match));
 	static char errorbuf[512];
@@ -438,7 +438,7 @@ Match *unreal_create_match(MatchType type, char *str, char **error)
  * @returns 1 if matched, 0 if not.
  * @note These (more logical) return values are opposite to the match_simple() function.
  */
-int unreal_match(Match *m, char *str)
+int unreal_match(Match *m, const char *str)
 {
 	if (m->type == MATCH_SIMPLE)
 	{
@@ -463,7 +463,7 @@ int unreal_match(Match *m, char *str)
 	return 0;
 }
 
-int unreal_match_method_strtoval(char *str)
+int unreal_match_method_strtoval(const char *str)
 {
 	if (!strcmp(str, "regex") || !strcmp(str, "pcre"))
 		return MATCH_PCRE_REGEX;
@@ -489,10 +489,11 @@ char *unreal_match_method_valtostr(int val)
  * Moved here from the censor channel and user mode module
  * (previously was present in both modules, code duplication)
  */
-int fast_badword_match(ConfigItem_badword *badword, char *line)
+int fast_badword_match(ConfigItem_badword *badword, const char *line)
 {
-	char *p;
+	const char *p;
 	int bwlen = strlen(badword->word);
+
 	if ((badword->type & BADW_TYPE_FAST_L) && (badword->type & BADW_TYPE_FAST_R))
 		return (our_strcasestr(line, badword->word) ? 1 : 0);
 
@@ -523,21 +524,20 @@ next:
  * buf is used for the result and max is sizeof(buf).
  * Assumptions[!]: max > 0 AND max > strlen(line)+1
  */
-int fast_badword_replace(ConfigItem_badword *badword, char *line, char *buf, int max)
+int fast_badword_replace(ConfigItem_badword *badword, const char *line, char *buf, int max)
 {
 	/* Some aliases ;P */
 	char *replacew = badword->replace ? badword->replace : REPLACEWORD;
-	char *pold = line, *pnew = buf; /* Pointers to old string and new string */
-	char *poldx = line;
+	const char *pold = line; /* pointer to the old string */
+	const char *poldx = line;
+	char *pnew = buf; /* pointer to the new string */
 	int replacen = -1; /* Only calculated if needed. w00t! saves us a few nanosecs? lol */
 	int searchn = -1;
-	char *startw, *endw;
+	const char *startw, *endw; /* start and end of the word */
 	char *c_eol = buf + max - 1; /* Cached end of (new) line */
 	int run = 1;
 	int cleaned = 0;
 
-	Debug((DEBUG_NOTICE, "replacing %s -> %s in '%s'", badword->word, replacew, line));
-
 	while(run) {
 		pold = our_strcasestr(pold, badword->word);
 		if (!pold)
@@ -617,7 +617,7 @@ int fast_badword_replace(ConfigItem_badword *badword, char *line, char *buf, int
  * the loadbadwords() function.  It's primary use is to filter swearing
  * in both private and public messages
  */
-char *stripbadwords(char *str, ConfigItem_badword *start_bw, int *blocked)
+const char *stripbadwords(const char *str, ConfigItem_badword *start_bw, int *blocked)
 {
 	static char cleanstr[4096];
 	char buf[4096];
@@ -692,14 +692,17 @@ char *stripbadwords(char *str, ConfigItem_badword *start_bw, int *blocked)
 					ret = pcre2_match(this_word->pcre2_expr, ptr, PCRE2_ZERO_TERMINATED, 0, 0, md, NULL); /* run the regex */
 					if (ret > 0)
 					{
-						ircd_log(LOG_ERROR, "pcre2_get_ovector_count: %d", pcre2_get_ovector_count(md));
 						dd = pcre2_get_ovector_pointer(md);
 						start = (int)dd[0];
 						end = (int)dd[1];
 						if ((start < 0) || (end < 0) || (start > strlen(ptr)) || (end > strlen(ptr)+1))
 						{
-							ircd_log(LOG_ERROR, "pcre2_match() returned an ovector with OOB start/end: %d/%d, str (%d): '%s'",
-								(int)start, (int)end, (int)strlen(ptr), ptr);
+							unreal_log(ULOG_FATAL, "main", "BUG_STRIPBADWORDS_PCRE2_MATCH_OOB", NULL,
+							           "[BUG] pcre2_match() returned an ovector with OOB start/end: $start/$end, len $length: '$buf'",
+							           log_data_integer("start", start),
+							           log_data_integer("end", end),
+							           log_data_integer("length", strlen(ptr)),
+							           log_data_string("buf", ptr));
 							abort();
 						}
 						m = end - start;
@@ -743,10 +746,10 @@ char *stripbadwords(char *str, ConfigItem_badword *start_bw, int *blocked)
  * if check_broadness is 1, the function will attempt to determine
  * if the given regex string is too broad (i.e. matches everything)
  */
-char *badword_config_check_regex(char *str, int fastsupport, int check_broadness)
+const char *badword_config_check_regex(const char *str, int fastsupport, int check_broadness)
 {
 	int regex=0;
-	char *tmp;
+	const char *tmp;
 	static char errorbuf[512];
 
 	if (fastsupport)
@@ -787,9 +790,9 @@ char *badword_config_check_regex(char *str, int fastsupport, int check_broadness
 	return NULL;
 }
 
-int badword_config_process(ConfigItem_badword *ca, char *str)
+int badword_config_process(ConfigItem_badword *ca, const char *str)
 {
-	char *tmp;
+	const char *tmp;
 	short regex = 0;
 	int ast_l = 0, ast_r = 0;
 
