@@ -19,19 +19,13 @@
 
 #include "unrealircd.h"
 
-ModuleHeader MOD_HEADER(authprompt)
+ModuleHeader MOD_HEADER
 = {
 	"authprompt",
 	"1.0",
 	"SASL authentication for clients that don't support SASL",
-	"3.2-b8-1",
-	NULL 
-};
-
-typedef struct _multiline MultiLine;
-struct _multiline {
-	MultiLine *prev, *next;
-	char *line;
+	"UnrealIRCd Team",
+	"unrealircd-6",
 };
 
 /** Configuration settings */
@@ -39,11 +33,12 @@ struct {
 	int enabled;
 	MultiLine *message;
 	MultiLine *fail_message;
+	MultiLine *unconfirmed_message;
 } cfg;
 
 /** User struct */
-typedef struct _apuser APUser;
-struct _apuser {
+typedef struct APUser APUser;
+struct APUser {
 	char *authmsg;
 };
 
@@ -56,13 +51,12 @@ static void init_config(void);
 static void config_postdefaults(void);
 int authprompt_config_test(ConfigFile *, ConfigEntry *, int, int *);
 int authprompt_config_run(ConfigFile *, ConfigEntry *, int);
-int authprompt_require_sasl(aClient *acptr, char *reason);
-int authprompt_sasl_continuation(aClient *acptr, char *buf);
-int authprompt_sasl_result(aClient *acptr, int success);
-int authprompt_place_host_ban(aClient *sptr, int action, char *reason, long duration);
-int authprompt_find_tkline_match(aClient *sptr, aTKline *tk);
-int authprompt_pre_connect(aClient *sptr);
-CMD_FUNC(m_auth);
+int authprompt_sasl_continuation(Client *client, const char *buf);
+int authprompt_sasl_result(Client *client, int success);
+int authprompt_place_host_ban(Client *client, int action, const char *reason, long duration);
+int authprompt_find_tkline_match(Client *client, TKL *tk);
+int authprompt_pre_connect(Client *client);
+CMD_FUNC(cmd_auth);
 void authprompt_md_free(ModData *md);
 
 /* Some macros */
@@ -70,13 +64,13 @@ void authprompt_md_free(ModData *md);
 #define SEUSER(x)       ((APUser *)moddata_client(x, authprompt_md).ptr)
 #define AGENT_SID(agent_p)      (agent_p->user != NULL ? agent_p->user->server : agent_p->name)
 
-MOD_TEST(authprompt)
+MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, authprompt_config_test);
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(authprompt)
+MOD_INIT()
 {
 	ModDataInfo mreq;
 
@@ -95,7 +89,6 @@ MOD_INIT(authprompt)
 
 	init_config();
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, authprompt_config_run);
-	HookAdd(modinfo->handle, HOOKTYPE_REQUIRE_SASL, 0, authprompt_require_sasl);
 	HookAdd(modinfo->handle, HOOKTYPE_SASL_CONTINUATION, 0, authprompt_sasl_continuation);
 	HookAdd(modinfo->handle, HOOKTYPE_SASL_RESULT, 0, authprompt_sasl_result);
 	HookAdd(modinfo->handle, HOOKTYPE_PLACE_HOST_BAN, 0, authprompt_place_host_ban);
@@ -105,17 +98,17 @@ MOD_INIT(authprompt)
 	 * a chance to be handled first.
 	 */
 	HookAdd(modinfo->handle, HOOKTYPE_PRE_LOCAL_CONNECT, -1000000, authprompt_pre_connect);
-	CommandAdd(modinfo->handle, "AUTH", m_auth, 1, M_UNREGISTERED);
+	CommandAdd(modinfo->handle, "AUTH", cmd_auth, 1, CMD_UNREGISTERED);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(authprompt)
+MOD_LOAD()
 {
 	config_postdefaults();
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(authprompt)
+MOD_UNLOAD()
 {
 	free_config();
 	return MOD_SUCCESS;
@@ -128,24 +121,6 @@ static void init_config(void)
 	cfg.enabled = 0;
 }
 
-static void addmultiline(MultiLine **l, char *line)
-{
-	MultiLine *m = MyMallocEx(sizeof(MultiLine));
-	m->line = strdup(line);
-	append_ListItem((ListStruct *)m, (ListStruct **)l);
-}
-
-static void freemultiline(MultiLine *l)
-{
-	MultiLine *l_next;
-	for (; l; l = l_next)
-	{
-		l_next = l->next;
-		safefree(l->line);
-		MyFree(l);
-	}
-}
-
 static void config_postdefaults(void)
 {
 	if (!cfg.message)
@@ -157,12 +132,18 @@ static void config_postdefaults(void)
 	{
 		addmultiline(&cfg.fail_message, "Authentication failed.");
 	}
+	if (!cfg.unconfirmed_message)
+	{
+		addmultiline(&cfg.unconfirmed_message, "You are trying to use an unconfirmed services account.");
+		addmultiline(&cfg.unconfirmed_message, "This services account can only be used after it has been activated/confirmed.");
+	}
 }
 
 static void free_config(void)
 {
 	freemultiline(cfg.message);
 	freemultiline(cfg.fail_message);
+	freemultiline(cfg.unconfirmed_message);
 	memset(&cfg, 0, sizeof(cfg)); /* needed! */
 }
 
@@ -175,29 +156,32 @@ int authprompt_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 		return 0;
 
 	/* We are only interrested in set::authentication-prompt... */
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "authentication-prompt"))
+	if (!ce || !ce->name || strcmp(ce->name, "authentication-prompt"))
 		return 0;
 
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!cep->ce_vardata)
+		if (!cep->value)
 		{
 			config_error("%s:%i: set::authentication-prompt::%s with no value",
-				cep->ce_fileptr->cf_filename, cep->ce_varlinenum, cep->ce_varname);
+				cep->file->filename, cep->line_number, cep->name);
 			errors++;
 		} else
-		if (!strcmp(cep->ce_varname, "enabled"))
+		if (!strcmp(cep->name, "enabled"))
 		{
 		} else
-		if (!strcmp(cep->ce_varname, "message"))
+		if (!strcmp(cep->name, "message"))
 		{
 		} else
-		if (!strcmp(cep->ce_varname, "fail-message"))
+		if (!strcmp(cep->name, "fail-message"))
+		{
+		} else
+		if (!strcmp(cep->name, "unconfirmed-message"))
 		{
 		} else
 		{
 			config_error("%s:%i: unknown directive set::authentication-prompt::%s",
-				cep->ce_fileptr->cf_filename, cep->ce_varlinenum, cep->ce_varname);
+				cep->file->filename, cep->line_number, cep->name);
 			errors++;
 		}
 	}
@@ -213,22 +197,26 @@ int authprompt_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		return 0;
 
 	/* We are only interrested in set::authentication-prompt... */
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "authentication-prompt"))
+	if (!ce || !ce->name || strcmp(ce->name, "authentication-prompt"))
 		return 0;
 
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
+	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->ce_varname, "enabled"))
+		if (!strcmp(cep->name, "enabled"))
 		{
-			cfg.enabled = config_checkval(cep->ce_vardata, CFG_YESNO);
+			cfg.enabled = config_checkval(cep->value, CFG_YESNO);
 		} else
-		if (!strcmp(cep->ce_varname, "message"))
+		if (!strcmp(cep->name, "message"))
 		{
-			addmultiline(&cfg.message, cep->ce_vardata);
+			addmultiline(&cfg.message, cep->value);
 		} else
-		if (!strcmp(cep->ce_varname, "fail-message"))
+		if (!strcmp(cep->name, "fail-message"))
 		{
-			addmultiline(&cfg.fail_message, cep->ce_vardata);
+			addmultiline(&cfg.fail_message, cep->value);
+		} else
+		if (!strcmp(cep->name, "unconfirmed-message"))
+		{
+			addmultiline(&cfg.unconfirmed_message, cep->value);
 		}
 	}
 	return 1;
@@ -240,8 +228,8 @@ void authprompt_md_free(ModData *md)
 
 	if (se)
 	{
-		safefree(se->authmsg);
-		MyFree(se);
+		safe_free(se->authmsg);
+		safe_free(se);
 		md->ptr = se = NULL;
 	}
 }
@@ -251,7 +239,7 @@ void authprompt_md_free(ModData *md)
  * @param username Pointer to the username string.
  * @param password Pointer to the password string.
  * @retval 1 if the format is correct, 0 if not.
- * @notes The returned 'username' and 'password' are valid until next call to parse_nickpass().
+ * @note The returned 'username' and 'password' are valid until next call to parse_nickpass().
  */
 int parse_nickpass(const char *str, char **username, char **password)
 {
@@ -274,20 +262,6 @@ int parse_nickpass(const char *str, char **username, char **password)
 	return 1;
 }
 
-/* NOTE: This function is stolen from m_sasl. Not good. */
-static const char *encode_puid(aClient *client)
-{
-	static char buf[HOSTLEN + 20];
-
-	/* create a cookie if necessary (and in case getrandom16 returns 0, then run again) */
-	while (!client->local->sasl_cookie)
-		client->local->sasl_cookie = getrandom16();
-
-	snprintf(buf, sizeof buf, "%s!0.%d", me.name, client->local->sasl_cookie);
-
-	return buf;
-}
-
 char *make_authbuf(const char *username, const char *password)
 {
 	char inbuf[256];
@@ -295,7 +269,7 @@ char *make_authbuf(const char *username, const char *password)
 	int size;
 
 	size = strlen(username) + 1 + strlen(username) + 1 + strlen(password);
-	if (size >= sizeof(inbuf))
+	if (size >= sizeof(inbuf)-1)
 		return NULL; /* too long */
 
 	/* Because size limits are already checked above, we can cut some corners here: */
@@ -315,208 +289,191 @@ char *make_authbuf(const char *username, const char *password)
  * Among other things, this is used to discover the agent
  * which will later be used for this session.
  */
-void send_first_auth(aClient *sptr)
+void send_first_auth(Client *client)
 {
-	aClient *acptr;
-	char *addr = BadPtr(sptr->ip) ? "0" : sptr->ip;
-	char *certfp = moddata_client_get(sptr, "certfp");
-	acptr = find_client(SASL_SERVER, NULL);
-	if (!acptr)
+	Client *sasl_server;
+	char *addr = BadPtr(client->ip) ? "0" : client->ip;
+	const char *certfp = moddata_client_get(client, "certfp");
+	sasl_server = find_client(SASL_SERVER, NULL);
+	if (!sasl_server)
 	{
 		/* Services down. */
 		return;
 	}
 
-	sendto_one(acptr, ":%s SASL %s %s H %s %s",
-	    me.name, SASL_SERVER, encode_puid(sptr), addr, addr);
+	sendto_one(sasl_server, NULL, ":%s SASL %s %s H %s %s",
+	    me.id, SASL_SERVER, client->id, addr, addr);
 
 	if (certfp)
-		sendto_one(acptr, ":%s SASL %s %s S %s %s",
-		    me.name, SASL_SERVER, encode_puid(sptr), "PLAIN", certfp);
+		sendto_one(sasl_server, NULL, ":%s SASL %s %s S %s %s",
+		    me.id, SASL_SERVER, client->id, "PLAIN", certfp);
 	else
-		sendto_one(acptr, ":%s SASL %s %s S %s",
-		    me.name, SASL_SERVER, encode_puid(sptr), "PLAIN");
+		sendto_one(sasl_server, NULL, ":%s SASL %s %s S %s",
+		    me.id, SASL_SERVER, client->id, "PLAIN");
 
 	/* The rest is sent from authprompt_sasl_continuation() */
 
-	sptr->local->sasl_out++;
+	client->local->sasl_out++;
 }
 
-CMD_FUNC(m_auth)
+CMD_FUNC(cmd_auth)
 {
 	char *username = NULL;
 	char *password = NULL;
 	char *authbuf;
 
-	if (!SEUSER(sptr))
+	if (!SEUSER(client))
 	{
-		if (CHECKPROTO(sptr, PROTO_SASL))
-			sendnotice(sptr, "ERROR: Cannot use /AUTH when your client is doing SASL.");
+		if (HasCapability(client, "sasl"))
+			sendnotice(client, "ERROR: Cannot use /AUTH when your client is doing SASL.");
 		else
-			sendnotice(sptr, "ERROR: /AUTH authentication request received before authentication prompt (too early!)");
-		return 0;
+			sendnotice(client, "ERROR: /AUTH authentication request received before authentication prompt (too early!)");
+		return;
 	}
 
 	if ((parc < 2) || BadPtr(parv[1]) || !parse_nickpass(parv[1], &username, &password))
 	{
-		sendnotice(sptr, "ERROR: Syntax is: /AUTH <nickname>:<password>");
-		sendnotice(sptr, "Example: /AUTH mynick:secretpass");
-		return 0;
+		sendnotice(client, "ERROR: Syntax is: /AUTH <nickname>:<password>");
+		sendnotice(client, "Example: /AUTH mynick:secretpass");
+		return;
 	}
 
 	if (!SASL_SERVER)
 	{
-		sendnotice(sptr, "ERROR: SASL is not configured on this server, or services are down.");
+		sendnotice(client, "ERROR: SASL is not configured on this server, or services are down.");
 		// numeric instead? SERVICESDOWN?
-		return 0;
+		return;
 	}
 
 	/* Presumably if the user is really fast, this could happen.. */
-	if (*sptr->local->sasl_agent || SEUSER(sptr)->authmsg)
+	if (*client->local->sasl_agent || SEUSER(client)->authmsg)
 	{
-		sendnotice(sptr, "ERROR: Previous authentication request is still in progress. Please wait.");
-		return 0;
+		sendnotice(client, "ERROR: Previous authentication request is still in progress. Please wait.");
+		return;
 	}
 
 	authbuf = make_authbuf(username, password);
 	if (!authbuf)
 	{
-		sendnotice(sptr, "ERROR: Internal error. Oversized username/password?");
-		return 0;
+		sendnotice(client, "ERROR: Internal error. Oversized username/password?");
+		return;
 	}
 
-	safestrdup(SEUSER(sptr)->authmsg, authbuf);
+	safe_strdup(SEUSER(client)->authmsg, authbuf);
 
-	send_first_auth(sptr);
-
-	return 0;
+	send_first_auth(client);
 }
 
-void send_multinotice(aClient *sptr, MultiLine *m)
-{
-	for (; m; m = m->next)
-		sendnotice(sptr, "%s", m->line);
-}
-
-void authprompt_tag_as_auth_required(aClient *sptr)
+void authprompt_tag_as_auth_required(Client *client)
 {
 	/* Allocate, and therefore indicate, that we are going to handle SASL for this user */
-	if (!SEUSER(sptr))
-		SetAPUser(sptr, MyMallocEx(sizeof(APUser)));
+	if (!SEUSER(client))
+		SetAPUser(client, safe_alloc(sizeof(APUser)));
 }
 
-void authprompt_send_auth_required_message(aClient *sptr)
+void authprompt_send_auth_required_message(Client *client)
 {
 	/* Display set::authentication-prompt::message */
-	send_multinotice(sptr, cfg.message);
-}
-
-int authprompt_require_sasl(aClient *sptr, char *reason)
-{
-	/* If the client did SASL then we (authprompt) will not kick in */
-	if (CHECKPROTO(sptr, PROTO_SASL))
-		return 0;
-
-	authprompt_tag_as_auth_required(sptr);
-
-	/* Display the require authentication::reason */
-	if (reason)
-		sendnotice(sptr, "%s", reason);
-
-	authprompt_send_auth_required_message(sptr);
-
-	return 1;
+	sendnotice_multiline(client, cfg.message);
 }
 
 /* Called upon "place a host ban on this user" (eg: spamfilter, blacklist, ..) */
-int authprompt_place_host_ban(aClient *sptr, int action, char *reason, long duration)
+int authprompt_place_host_ban(Client *client, int action, const char *reason, long duration)
 {
 	/* If it's a soft-xx action and the user is not logged in
 	 * and the user is not yet online, then we will handle this user.
 	 */
-	if (IsSoftBanAction(action) && !IsLoggedIn(sptr) && !IsPerson(sptr))
+	if (IsSoftBanAction(action) && !IsLoggedIn(client) && !IsUser(client))
 	{
 		/* Send ban reason */
 		if (reason)
-			sendnotice(sptr, "%s", reason);
+			sendnotice(client, "%s", reason);
 
 		/* And tag the user */
-		authprompt_tag_as_auth_required(sptr);
-		return 0; /* pretend user is exempt */
+		authprompt_tag_as_auth_required(client);
+		authprompt_send_auth_required_message(client);
+		return 1; /* pretend user is killed */
 	}
 	return 99; /* no action taken, proceed normally */
 }
 
 /** Called upon "check for KLINE/GLINE" */
-int authprompt_find_tkline_match(aClient *sptr, aTKline *tk)
+int authprompt_find_tkline_match(Client *client, TKL *tkl)
 {
 	/* If it's a soft-xx action and the user is not logged in
 	 * and the user is not yet online, then we will handle this user.
 	 */
-	if ((tk->subtype & TKL_SUBTYPE_SOFT) && !IsLoggedIn(sptr) && !IsPerson(sptr))
+	if (TKLIsServerBan(tkl) &&
+	   (tkl->ptr.serverban->subtype & TKL_SUBTYPE_SOFT) &&
+	   !IsLoggedIn(client) &&
+	   !IsUser(client))
 	{
 		/* Send ban reason */
-		if (tk->reason)
-			sendnotice(sptr, "%s", tk->reason);
+		if (tkl->ptr.serverban->reason)
+			sendnotice(client, "%s", tkl->ptr.serverban->reason);
 
 		/* And tag the user */
-		authprompt_tag_as_auth_required(sptr);
-		return 0; /* pretend user is exempt */
+		authprompt_tag_as_auth_required(client);
+		authprompt_send_auth_required_message(client);
+		return 1; /* pretend user is killed */
 	}
 	return 99; /* no action taken, proceed normally */
 }
 
-int authprompt_pre_connect(aClient *sptr)
+int authprompt_pre_connect(Client *client)
 {
 	/* If the user is tagged as auth required and not logged in, then.. */
-	if (SEUSER(sptr) && !IsLoggedIn(sptr))
+	if (SEUSER(client) && !IsLoggedIn(client))
 	{
-		authprompt_send_auth_required_message(sptr);
-		return -1; /* do not process register_user() */
+		authprompt_send_auth_required_message(client);
+		return HOOK_DENY; /* do not process register_user() */
 	}
 
-	return 0; /* no action taken, proceed normally */
+	return HOOK_CONTINUE; /* no action taken, proceed normally */
 }
 
-int authprompt_sasl_continuation(aClient *sptr, char *buf)
+int authprompt_sasl_continuation(Client *client, const char *buf)
 {
 	/* If it's not for us (eg: user is doing real SASL) then return 0. */
-	if (!SEUSER(sptr) || !SEUSER(sptr)->authmsg)
+	if (!SEUSER(client) || !SEUSER(client)->authmsg)
 		return 0;
 
 	if (!strcmp(buf, "+"))
 	{
-		aClient *agent = find_client(sptr->local->sasl_agent, NULL);
+		Client *agent = find_client(client->local->sasl_agent, NULL);
 		if (agent)
 		{
-			sendto_one(agent, ":%s SASL %s %s C %s",
-				me.name, AGENT_SID(agent), encode_puid(sptr), SEUSER(sptr)->authmsg);
+			sendto_one(agent, NULL, ":%s SASL %s %s C %s",
+				me.id, AGENT_SID(agent), client->id, SEUSER(client)->authmsg);
 		}
-		SEUSER(sptr)->authmsg = NULL;
+		safe_free(SEUSER(client)->authmsg);
 	}
 	return 1; /* inhibit displaying of message */
 }
 
-int authprompt_sasl_result(aClient *sptr, int success)
+int authprompt_sasl_result(Client *client, int success)
 {
 	/* If it's not for us (eg: user is doing real SASL) then return 0. */
-	if (!SEUSER(sptr))
+	if (!SEUSER(client))
 		return 0;
 
 	if (!success)
 	{
-		send_multinotice(sptr, cfg.fail_message);
+		sendnotice_multiline(client, cfg.fail_message);
+		return 1;
+	}
+
+	if (client->user && !IsLoggedIn(client))
+	{
+		sendnotice_multiline(client, cfg.unconfirmed_message);
 		return 1;
 	}
 
 	/* Authentication was a success */
-	if (*sptr->name && sptr->user && *sptr->user->username && IsNotSpoof(sptr))
+	if (*client->name && client->user && *client->user->username && IsNotSpoof(client))
 	{
-		register_user(sptr, sptr, sptr->name, sptr->user->username, NULL, NULL, NULL);
-		/* NOTE: register_user() may return FLUSH_BUFFER here, but since the caller
-		 * won't continue processing (won't touch 'sptr') it's safe.
-		 * That is, as long as we 'return 1'.
-		 */
+		register_user(client);
+		/* User MAY be killed now. But since we 'return 1' below, it's safe */
 	}
 
 	return 1; /* inhibit success/failure message */
