@@ -11,7 +11,7 @@ ModuleHeader MOD_HEADER
 	"5.0", /* Version */
 	"delayed join (+D,+d)", /* Short description of module */
 	"UnrealIRCd Team",
-	"unrealircd-5",
+	"unrealircd-6",
     };
 
 #define MOD_DATA_STR "delayjoin"
@@ -26,14 +26,14 @@ static Cmode_t EXTMODE_POST_DELAYED;
 int visible_in_channel(Client *client, Channel *channel);
 int moded_check_part(Client *client, Channel *channel);
 int moded_join(Client *client, Channel *channel);
-int moded_part(Client *client, Channel *channel, MessageTag *mtags, char *comment);
-int moded_quit(Client *client, MessageTag *mtags, char *comment);
-int deny_all(Client *client, Channel *channel, char mode, char *para, int checkt, int what);
+int moded_part(Client *client, Channel *channel, MessageTag *mtags, const char *comment);
+int moded_quit(Client *client, MessageTag *mtags, const char *comment);
+int delayjoin_is_ok(Client *client, Channel *channel, char mode, const char *para, int checkt, int what);
 int moded_chanmode(Client *client, Channel *channel,
-                   MessageTag *mtags, char *modebuf, char *parabuf, time_t sendts, int samode);
-int moded_prechanmsg(Client *client, Channel *channel, MessageTag *mtags, char *text, SendType sendtype);
-char *moded_serialize(ModData *m);
-void moded_unserialize(char *str, ModData *m);
+                   MessageTag *mtags, const char *modebuf, const char *parabuf, time_t sendts, int samode);
+int moded_prechanmsg(Client *client, Channel *channel, MessageTag *mtags, const char *text, SendType sendtype);
+const char *moded_serialize(ModData *m);
+void moded_unserialize(const char *str, ModData *m);
 
 MOD_INIT()
 {
@@ -46,13 +46,13 @@ MOD_INIT()
 	memset(&req, 0, sizeof(req));
 	req.paracount = 0;
 	req.is_ok = extcmode_default_requirechop;
-	req.flag = 'D';
+	req.letter = 'D';
 	CmodeDelayed = CmodeAdd(modinfo->handle, req, &EXTMODE_DELAYED);
 
 	memset(&req, 0, sizeof(req));
 	req.paracount = 0;
-	req.is_ok = deny_all;
-	req.flag = 'd';
+	req.is_ok = delayjoin_is_ok;
+	req.letter = 'd';
 	req.local = 1;
 	CmodePostDelayed = CmodeAdd(modinfo->handle, req, &EXTMODE_POST_DELAYED);
 
@@ -101,10 +101,10 @@ void set_post_delayed(Channel *channel)
 {
 	MessageTag *mtags = NULL;
 
-	channel->mode.extmode |= EXTMODE_POST_DELAYED;
+	channel->mode.mode |= EXTMODE_POST_DELAYED;
 
 	new_message(&me, NULL, &mtags);
-	sendto_channel(channel, &me, NULL, 0, 0, SEND_LOCAL, mtags, ":%s MODE %s +d", me.name, channel->chname);
+	sendto_channel(channel, &me, NULL, 0, 0, SEND_LOCAL, mtags, ":%s MODE %s +d", me.name, channel->name);
 	free_message_tags(mtags);
 }
 
@@ -112,10 +112,10 @@ void clear_post_delayed(Channel *channel)
 {
 	MessageTag *mtags = NULL;
 
-	channel->mode.extmode &= ~EXTMODE_POST_DELAYED;
+	channel->mode.mode &= ~EXTMODE_POST_DELAYED;
 
 	new_message(&me, NULL, &mtags);
-	sendto_channel(channel, &me, NULL, 0, 0, SEND_LOCAL, mtags, ":%s MODE %s -d", me.name, channel->chname);
+	sendto_channel(channel, &me, NULL, 0, 0, SEND_LOCAL, mtags, ":%s MODE %s -d", me.name, channel->name);
 	free_message_tags(mtags);
 }
 
@@ -157,14 +157,14 @@ bool channel_has_invisible_users(Channel *channel)
 
 bool channel_is_post_delayed(Channel *channel)
 {
-	if (channel->mode.extmode & EXTMODE_POST_DELAYED)
+	if (channel->mode.mode & EXTMODE_POST_DELAYED)
 		return true;
 	return false;
 }
 
 bool channel_is_delayed(Channel *channel)
 {
-	if (channel->mode.extmode & EXTMODE_DELAYED)
+	if (channel->mode.mode & EXTMODE_DELAYED)
 		return true;
 	return false;
 }
@@ -200,7 +200,7 @@ void clear_user_invisible(Channel *channel, Client *client)
 		}
 	}
 
-	if (should_clear && (channel->mode.extmode & EXTMODE_POST_DELAYED))
+	if (should_clear && (channel->mode.mode & EXTMODE_POST_DELAYED))
 	{
 		clear_post_delayed(channel);
 	}
@@ -217,18 +217,18 @@ void clear_user_invisible_announce(Channel *channel, Client *client, MessageTag
 	clear_user_invisible(channel, client);
 
 	ircsnprintf(joinbuf, sizeof(joinbuf), ":%s!%s@%s JOIN %s",
-				client->name, client->user->username, GetHost(client), channel->chname);
+				client->name, client->user->username, GetHost(client), channel->name);
 
 	ircsnprintf(exjoinbuf, sizeof(exjoinbuf), ":%s!%s@%s JOIN %s %s :%s",
-		client->name, client->user->username, GetHost(client), channel->chname,
-		IsLoggedIn(client) ? client->user->svid : "*",
+		client->name, client->user->username, GetHost(client), channel->name,
+		IsLoggedIn(client) ? client->user->account : "*",
 		client->info);
 
-	new_message_special(client, recv_mtags, &mtags, ":%s JOIN %s", client->name, channel->chname);
+	new_message_special(client, recv_mtags, &mtags, ":%s JOIN %s", client->name, channel->name);
 	for (i = channel->members; i; i = i->next)
 	{
 		Client *acptr = i->client;
-		if (!is_skochanop(acptr, channel) && acptr != client && MyConnect(acptr))
+		if (!check_channel_access(acptr, channel, "hoaq") && acptr != client && MyConnect(acptr))
 		{
 			if (HasCapabilityFast(acptr, CAP_EXTENDED_JOIN))
 				sendto_one(acptr, mtags, "%s", exjoinbuf);
@@ -256,7 +256,7 @@ void set_user_invisible(Channel *channel, Client *client)
 }
 
 
-int deny_all(Client *client, Channel *channel, char mode, char *para, int checkt, int what)
+int delayjoin_is_ok(Client *client, Channel *channel, char mode, const char *para, int checkt, int what)
 {
 	return EX_ALWAYS_DENY;
 }
@@ -276,7 +276,7 @@ int moded_join(Client *client, Channel *channel)
 	return 0;
 }
 
-int moded_part(Client *client, Channel *channel, MessageTag *mtags, char *comment)
+int moded_part(Client *client, Channel *channel, MessageTag *mtags, const char *comment)
 {
 	if (channel_is_delayed(channel) || channel_is_post_delayed(channel))
 		clear_user_invisible(channel, client);
@@ -284,7 +284,7 @@ int moded_part(Client *client, Channel *channel, MessageTag *mtags, char *commen
 	return 0;
 }
 
-int moded_quit(Client *client, MessageTag *mtags, char *comment)
+int moded_quit(Client *client, MessageTag *mtags, const char *comment)
 {
 	Membership *membership;
 	Channel *channel;
@@ -300,7 +300,7 @@ int moded_quit(Client *client, MessageTag *mtags, char *comment)
 	return 0;
 }
 
-int moded_chanmode(Client *client, Channel *channel, MessageTag *recv_mtags, char *modebuf, char *parabuf, time_t sendts, int samode)
+int moded_chanmode(Client *client, Channel *channel, MessageTag *recv_mtags, const char *modebuf, const char *parabuf, time_t sendts, int samode)
 {
 	long CAP_EXTENDED_JOIN = ClientCapabilityBit("extended-join");
 
@@ -337,16 +337,16 @@ int moded_chanmode(Client *client, Channel *channel, MessageTag *recv_mtags, cha
 					if (moded_user_invisible(i->client, channel))
 					{
 						MessageTag *mtags = NULL;
-						new_message_special(i->client, recv_mtags, &mtags, ":%s JOIN %s", i->client->name, channel->chname);
+						new_message_special(i->client, recv_mtags, &mtags, ":%s JOIN %s", i->client->name, channel->name);
 						if (HasCapabilityFast(user, CAP_EXTENDED_JOIN))
 						{
 							sendto_one(user, mtags, ":%s!%s@%s JOIN %s %s :%s",
 							           i->client->name, i->client->user->username, GetHost(i->client),
-							           channel->chname,
-							           IsLoggedIn(i->client) ? i->client->user->svid : "*",
+							           channel->name,
+							           IsLoggedIn(i->client) ? i->client->user->account : "*",
 							           i->client->info);
 						} else {
-							sendto_one(user, mtags, ":%s!%s@%s JOIN :%s", i->client->name, i->client->user->username, GetHost(i->client), channel->chname);
+							sendto_one(user, mtags, ":%s!%s@%s JOIN :%s", i->client->name, i->client->user->username, GetHost(i->client), channel->name);
 						}
 						free_message_tags(mtags);
 					}
@@ -374,8 +374,8 @@ int moded_chanmode(Client *client, Channel *channel, MessageTag *recv_mtags, cha
 					if (moded_user_invisible(i->client, channel))
 					{
 						MessageTag *mtags = NULL;
-						new_message_special(i->client, recv_mtags, &mtags, ":%s PART %s", i->client->name, channel->chname);
-						sendto_one(user, mtags, ":%s!%s@%s PART :%s", i->client->name, i->client->user->username, GetHost(i->client), channel->chname);
+						new_message_special(i->client, recv_mtags, &mtags, ":%s PART %s", i->client->name, channel->name);
+						sendto_one(user, mtags, ":%s!%s@%s PART :%s", i->client->name, i->client->user->username, GetHost(i->client), channel->name);
 						free_message_tags(mtags);
 					}
 				}
@@ -387,7 +387,7 @@ int moded_chanmode(Client *client, Channel *channel, MessageTag *recv_mtags, cha
 	return 0;
 }
 
-int moded_prechanmsg(Client *client, Channel *channel, MessageTag *mtags, char *text, SendType sendtype)
+int moded_prechanmsg(Client *client, Channel *channel, MessageTag *mtags, const char *text, SendType sendtype)
 {
 	if ((channel_is_delayed(channel) || channel_is_post_delayed(channel)) && (moded_user_invisible(client, channel)))
 		clear_user_invisible_announce(channel, client, mtags);
@@ -395,12 +395,12 @@ int moded_prechanmsg(Client *client, Channel *channel, MessageTag *mtags, char *
 	return 0;
 }
 
-char *moded_serialize(ModData *m)
+const char *moded_serialize(ModData *m)
 {
 	return m->i ? "1" : "0";
 }
 
-void moded_unserialize(char *str, ModData *m)
+void moded_unserialize(const char *str, ModData *m)
 {
 	m->i = atoi(str);
 }
