@@ -21,58 +21,59 @@
 
 CMD_FUNC(nocolor);
 
-ModuleHeader MOD_HEADER(nocolor)
+ModuleHeader MOD_HEADER
   = {
 	"chanmodes/nocolor",
 	"4.2",
 	"Channel Mode +c",
-	"3.2-b8-1",
-	NULL 
+	"UnrealIRCd Team",
+	"unrealircd-6",
     };
 
 Cmode_t EXTCMODE_NOCOLOR;
 
-#define IsNoColor(chptr)    (chptr->mode.extmode & EXTCMODE_NOCOLOR)
+#define IsNoColor(channel)    (channel->mode.mode & EXTCMODE_NOCOLOR)
 
-char *nocolor_prechanmsg(aClient *sptr, aChannel *chptr, char *text, int notice);
-char *nocolor_prelocalpart(aClient *sptr, aChannel *chptr, char *comment);
-char *nocolor_prelocalquit(aClient *sptr, char *comment);
+int nocolor_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype);
+const char *nocolor_prelocalpart(Client *client, Channel *channel, const char *comment);
+const char *nocolor_prelocalquit(Client *client, const char *comment);
 
-MOD_TEST(nocolor)
+MOD_TEST()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_INIT(nocolor)
+MOD_INIT()
 {
 CmodeInfo req;
 
 	/* Channel mode */
 	memset(&req, 0, sizeof(req));
 	req.paracount = 0;
-	req.flag = 'c';
+	req.letter = 'c';
 	req.is_ok = extcmode_default_requirechop;
 	CmodeAdd(modinfo->handle, req, &EXTCMODE_NOCOLOR);
 	
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_CHANMSG, 0, nocolor_prechanmsg);
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_PART, 0, nocolor_prelocalpart);
-	HookAddPChar(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT, 0, nocolor_prelocalquit);
+	HookAdd(modinfo->handle, HOOKTYPE_CAN_SEND_TO_CHANNEL, 0, nocolor_can_send_to_channel);
+	HookAddConstString(modinfo->handle, HOOKTYPE_PRE_LOCAL_PART, 0, nocolor_prelocalpart);
+	HookAddConstString(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT_CHAN, 0, nocolor_prelocalpart);
+	HookAddConstString(modinfo->handle, HOOKTYPE_PRE_LOCAL_QUIT, 0, nocolor_prelocalquit);
 	
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	return MOD_SUCCESS;
 }
 
-MOD_LOAD(nocolor)
+MOD_LOAD()
 {
 	return MOD_SUCCESS;
 }
 
-MOD_UNLOAD(nocolor)
+MOD_UNLOAD()
 {
 	return MOD_SUCCESS;
 }
 
-static int IsUsingColor(char *s)
+static int IsUsingColor(const char *s)
 {
         if (!s)
                 return 0;
@@ -84,62 +85,57 @@ static int IsUsingColor(char *s)
         return 0;
 }
 
-char *nocolor_prechanmsg(aClient *sptr, aChannel *chptr, char *text, int notice)
+int nocolor_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype)
 {
 	Hook *h;
 	int i;
 
-	if (MyClient(sptr) && IsNoColor(chptr) && IsUsingColor(text))
+	if (MyUser(client) && IsNoColor(channel) && IsUsingColor(*msg))
 	{
 		for (h = Hooks[HOOKTYPE_CAN_BYPASS_CHANNEL_MESSAGE_RESTRICTION]; h; h = h->next)
 		{
-			i = (*(h->func.intfunc))(sptr, chptr, BYPASS_CHANMSG_COLOR);
+			i = (*(h->func.intfunc))(client, channel, BYPASS_CHANMSG_COLOR);
+			if (i == HOOK_ALLOW)
+				return HOOK_CONTINUE; /* bypass this restriction */
 			if (i != HOOK_CONTINUE)
 				break;
 		}
-		if (i == HOOK_ALLOW)
-			return text; /* bypass */
 
-		if (!notice)
-		{
-			sendto_one(sptr, err_str(ERR_CANNOTSENDTOCHAN),
-			           me.name, sptr->name, chptr->chname,
-			           "Color is not permitted in this channel", chptr->chname);
-		}
-		return NULL; /* block */
+		*errmsg = "Color is not permitted in this channel";
+		return HOOK_DENY;
 	}
 
-	return text;
+	return HOOK_CONTINUE;
 }
 
-char *nocolor_prelocalpart(aClient *sptr, aChannel *chptr, char *comment)
+const char *nocolor_prelocalpart(Client *client, Channel *channel, const char *comment)
 {
 	if (!comment)
 		return NULL;
 
-	if (MyClient(sptr) && IsNoColor(chptr) && IsUsingColor(comment))
+	if (MyUser(client) && IsNoColor(channel) && IsUsingColor(comment))
 		return NULL;
 
 	return comment;
 }
 
 /** Is any channel where the user is in +c? */
-static int IsAnyChannelNoColor(aClient *sptr)
+static int IsAnyChannelNoColor(Client *client)
 {
 	Membership *lp;
 
-	for (lp = sptr->user->channel; lp; lp = lp->next)
-		if (IsNoColor(lp->chptr))
+	for (lp = client->user->channel; lp; lp = lp->next)
+		if (IsNoColor(lp->channel))
 			return 1;
 	return 0;
 }
 
-char *nocolor_prelocalquit(aClient *sptr, char *comment)
+const char *nocolor_prelocalquit(Client *client, const char *comment)
 {
 	if (!comment)
 		return NULL;
 
-	if (MyClient(sptr) && !BadPtr(comment) && IsUsingColor(comment) && IsAnyChannelNoColor(sptr))
+	if (MyUser(client) && !BadPtr(comment) && IsUsingColor(comment) && IsAnyChannelNoColor(client))
 		return NULL;
 
 	return comment;
