@@ -32,7 +32,7 @@ ModuleHeader MOD_HEADER
 	"5.0", /* Version */
 	"Character System (set::allowed-nickchars)", /* Short description of module */
 	"UnrealIRCd Team",
-	"unrealircd-5",
+	"unrealircd-6",
 };
 
 /* NOTE: it is guaranteed that char is unsigned by compiling options
@@ -84,7 +84,6 @@ struct LangList
 
 /* MUST be alphabetized (first column) */
 static LangList langlist[] = {
-/*	{ "arabic",       "ara", LANGAV_ASCII|LANGAV_ISO8859_6 }, -- TODO: check if this has issues first! */
 	{ "arabic-utf8", "ara-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_ARABIC_UTF8 },
 	{ "belarussian-utf8", "blr-utf8", LANGAV_ASCII|LANGAV_UTF8|LANGAV_CYRILLIC_UTF8 },
 	{ "belarussian-w1251", "blr", LANGAV_ASCII|LANGAV_W1251 },
@@ -191,7 +190,7 @@ MOD_TEST()
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	EfunctionAdd(modinfo->handle, EFUNC_DO_NICK_NAME, _do_nick_name);
 	EfunctionAdd(modinfo->handle, EFUNC_DO_REMOTE_NICK_NAME, _do_remote_nick_name);
-	EfunctionAddPChar(modinfo->handle, EFUNC_CHARSYS_GET_CURRENT_LANGUAGES, _charsys_get_current_languages);
+	EfunctionAddString(modinfo->handle, EFUNC_CHARSYS_GET_CURRENT_LANGUAGES, _charsys_get_current_languages);
 	charsys_reset();
 	charsys_reset_pretest();
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, charsys_config_test);
@@ -228,26 +227,26 @@ int charsys_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 		return 0;
 
 	/* We are only interrested in set::allowed-nickchars... */
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "allowed-nickchars"))
+	if (!ce || !ce->name || strcmp(ce->name, "allowed-nickchars"))
 		return 0;
 
-	if (ce->ce_vardata)
+	if (ce->value)
 	{
 		config_error("%s:%i: set::allowed-nickchars: please use 'allowed-nickchars { name; };' "
 					 "and not 'allowed-nickchars name;'",
-					 ce->ce_fileptr->cf_filename, ce->ce_varlinenum);
+					 ce->file->filename, ce->line_number);
 		/* Give up immediately. Don't bother the user with any other errors. */
 		errors++;
 		*errs = errors;
 		return -1;
 	}
 
-	for (cep = ce->ce_entries; cep; cep=cep->ce_next)
+	for (cep = ce->items; cep; cep=cep->next)
 	{
-		if (!charsys_test_language(cep->ce_varname))
+		if (!charsys_test_language(cep->name))
 		{
 			config_error("%s:%i: set::allowed-nickchars: Unknown (sub)language '%s'",
-				ce->ce_fileptr->cf_filename, ce->ce_varlinenum, cep->ce_varname);
+				ce->file->filename, ce->line_number, cep->name);
 			errors++;
 		}
 	}
@@ -264,11 +263,11 @@ int charsys_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		return 0;
 
 	/* We are only interrested in set::allowed-nickchars... */
-	if (!ce || !ce->ce_varname || strcmp(ce->ce_varname, "allowed-nickchars"))
+	if (!ce || !ce->name || strcmp(ce->name, "allowed-nickchars"))
 		return 0;
 
-	for (cep = ce->ce_entries; cep; cep = cep->ce_next)
-		charsys_add_language(cep->ce_varname);
+	for (cep = ce->items; cep; cep = cep->next)
+		charsys_add_language(cep->name);
 
 	return 1;
 }
@@ -886,13 +885,6 @@ void charsys_add_language(char *name)
 		charsys_addmultibyterange(0xc3, 0xc3, 0xba, 0xba);
 		charsys_addmultibyterange(0xc3, 0xc3, 0xbd, 0xbe);
 	}
-/*	if (latin1 || !strcmp(name, "arabic")) -- Since when is arabic considered latin(1)??? oh man...
-	{
-		char bytes[] = { 0xa0, 0xa4, 0xac, 0xad, 0xbb, 0xbf, 0x00 };
-		charsys_addallowed(bytes);
-		charsys_addallowed_range(0xc1, 0xda);
-		charsys_addallowed_range(0xe0, 0xf2);
-	} */
 
 	/* [LATIN2] and rest of [LATIN-UTF8] */
 	/* actually hungarian is a special case, include it in both w1250 and latin2 ;p */
