@@ -24,24 +24,24 @@ ModuleHeader MOD_HEADER
 	"4.2",
 	"ExtBan ~m - bypass +m/+n/+c/+S/+T (msgbypass)",
 	"UnrealIRCd Team",
-	"unrealircd-5",
+	"unrealircd-6",
 };
 
 /* Forward declarations */
-int extban_msgbypass_is_banned(Client *client, Channel *channel, char *banin, int type, char **msg, char **errmsg);
 int msgbypass_can_bypass(Client *client, Channel *channel, BypassChannelMessageRestrictionType bypass_type);
-int msgbypass_extban_is_ok(Client *client, Channel* channel, char *para, int checkt, int what, int what2);
-char *msgbypass_extban_conv_param(char *para);
+int msgbypass_extban_is_ok(BanContext *b);
+const char *msgbypass_extban_conv_param(BanContext *b, Extban *extban);
 
 /** Called upon module init */
 MOD_INIT()
 {
 	ExtbanInfo req;
 	
-	req.flag = 'm';
+	memset(&req, 0, sizeof(req));
+	req.letter = 'm';
+	req.name = "msgbypass";
 	req.is_ok = msgbypass_extban_is_ok;
 	req.conv_param = msgbypass_extban_conv_param;
-	req.is_banned = extban_msgbypass_is_banned;
 	req.options = EXTBOPT_ACTMODIFIER;
 	if (!ExtbanAdd(modinfo->handle, req))
 	{
@@ -67,42 +67,50 @@ MOD_UNLOAD()
 	return MOD_SUCCESS;
 }
 
-/** Is the user banned? No, never by us anyway. */
-int extban_msgbypass_is_banned(Client *client, Channel *channel, char *banin, int type, char **msg, char **errmsg)
-{
-	return 0; /* not banned by us */
-}
-
 /** Can the user bypass restrictions? */
 int msgbypass_can_bypass(Client *client, Channel *channel, BypassChannelMessageRestrictionType bypass_type)
 {
 	Ban *ban;
 	char *p;
-	
+	BanContext *b = safe_alloc(sizeof(BanContext));
+
+	b->client = client;
+	b->channel = channel;
+	b->ban_check_types = BANCHK_MSG;
+
 	for (ban = channel->exlist; ban; ban=ban->next)
 	{
+		char *type;
+		char *matchby;
+
 		if (!strncmp(ban->banstr, "~m:", 3))
+			type = ban->banstr + 3;
+		else if (!strncmp(ban->banstr, "~msgbypass:", 11))
+			type = ban->banstr + 11;
+		else
+			continue;
+
+		if (((bypass_type == BYPASS_CHANMSG_EXTERNAL) && !strncmp(type, "external:", 9)) ||
+		    ((bypass_type == BYPASS_CHANMSG_MODERATED) && !strncmp(type, "moderated:", 10)) ||
+		    ((bypass_type == BYPASS_CHANMSG_COLOR) && !strncmp(type, "color:", 6)) ||
+		    ((bypass_type == BYPASS_CHANMSG_CENSOR) && !strncmp(type, "censor:", 7)) ||
+		    ((bypass_type == BYPASS_CHANMSG_NOTICE) && !strncmp(type, "notice:", 7)))
 		{
-			char *type = ban->banstr + 3;
-			char *matchby;
+			matchby = strchr(type, ':');
+			if (!matchby)
+				continue;
+			matchby++;
 			
-			if (((bypass_type == BYPASS_CHANMSG_EXTERNAL) && !strncmp(type, "external:", 9)) ||
-			    ((bypass_type == BYPASS_CHANMSG_MODERATED) && !strncmp(type, "moderated:", 10)) ||
-			    ((bypass_type == BYPASS_CHANMSG_COLOR) && !strncmp(type, "color:", 6)) || 
-			    ((bypass_type == BYPASS_CHANMSG_CENSOR) && !strncmp(type, "censor:", 7)) ||
-			    ((bypass_type == BYPASS_CHANMSG_NOTICE) && !strncmp(type, "notice:", 7)))
+			b->banstr = matchby;
+			if (ban_check_mask(b))
 			{
-				matchby = strchr(type, ':');
-				if (!matchby)
-					continue;
-				matchby++;
-				
-				if (ban_check_mask(client, channel, matchby, BANCHK_MSG, NULL, NULL, 0))
-					return HOOK_ALLOW; /* Yes, user may bypass */
+				safe_free(b);
+				return HOOK_ALLOW; /* Yes, user may bypass */
 			}
 		}
 	}
 
+	safe_free(b);
 	return HOOK_CONTINUE; /* No, may NOT bypass. */
 }
 
@@ -121,16 +129,16 @@ int msgbypass_extban_type_ok(char *type)
 }
 
 #define MAX_LENGTH 128
-char *msgbypass_extban_conv_param(char *para_in)
+const char *msgbypass_extban_conv_param(BanContext *b, Extban *extban)
 {
 	static char retbuf[MAX_LENGTH+1];
 	char para[MAX_LENGTH+1];
 	char tmpmask[MAX_LENGTH+1];
 	char *type; /**< Type, such as 'external' */
 	char *matchby; /**< Matching method, such as 'n!u@h' */
-	char *newmask; /**< Cleaned matching method, such as 'n!u@h' */
+	const char *newmask; /**< Cleaned matching method, such as 'n!u@h' */
 
-	strlcpy(para, para_in+3, sizeof(para)); /* work on a copy (and truncate it) */
+	strlcpy(para, b->banstr, sizeof(para)); /* work on a copy (and truncate it) */
 	
 	/* ~m:type:n!u@h   for direct matching
 	 * ~m:type:~x:.... when calling another bantype
@@ -145,16 +153,13 @@ char *msgbypass_extban_conv_param(char *para_in)
 	if (!msgbypass_extban_type_ok(type))
 		return NULL;
 
-	/* This is quite silly, we have to create a fake extban here due to
-	 * the current API of extban_conv_param_nuh and extban_conv_param_nuh_or_extban
-	 * expecting the full banmask rather than the portion that actually matters.
-	 */
-	snprintf(tmpmask, sizeof(tmpmask), "~?:%s", matchby);
-	newmask = extban_conv_param_nuh_or_extban(tmpmask);
-	if (!newmask || (strlen(newmask) <= 3))
+	b->banstr = matchby;
+	newmask = extban_conv_param_nuh_or_extban(b, extban);
+	if (BadPtr(newmask))
 		return NULL;
 
-	snprintf(retbuf, sizeof(retbuf), "~m:%s:%s", type, newmask+3);
+	//snprintf(retbuf, sizeof(retbuf), "~m:%s:%s", type, newmask);
+	snprintf(retbuf, sizeof(retbuf), "%s:%s", type, newmask);
 	return retbuf;
 }
 
@@ -171,26 +176,25 @@ int msgbypass_extban_syntax(Client *client, int checkt, char *reason)
 	return 0; /* FAIL: ban rejected */
 }
 
-int msgbypass_extban_is_ok(Client *client, Channel* channel, char *para_in, int checkt, int what, int what2)
+int msgbypass_extban_is_ok(BanContext *b)
 {
-	char para[MAX_LENGTH+1];
-	char tmpmask[MAX_LENGTH+1];
+	static char para[MAX_LENGTH+1];
 	char *type; /**< Type, such as 'external' */
 	char *matchby; /**< Matching method, such as 'n!u@h' */
 	char *newmask; /**< Cleaned matching method, such as 'n!u@h' */
 
 	/* Always permit deletion */
-	if (what == MODE_DEL)
+	if (b->what == MODE_DEL)
 		return 1;
 	
-	if (what2 != EXBTYPE_EXCEPT)
+	if (b->ban_type != EXBTYPE_EXCEPT)
 	{
-		if (checkt == EXBCHK_PARAM)
-			sendnotice(client, "Ban type ~m only works with exceptions (+e) and not with bans or invex (+b/+I)");
+		if (b->is_ok_check == EXBCHK_PARAM)
+			sendnotice(b->client, "Ban type ~m only works with exceptions (+e) and not with bans or invex (+b/+I)");
 		return 0; /* reject */
 	}
 
-	strlcpy(para, para_in+3, sizeof(para)); /* work on a copy (and truncate it) */
+	strlcpy(para, b->banstr, sizeof(para)); /* work on a copy (and truncate it) */
 	
 	/* ~m:type:n!u@h   for direct matching
 	 * ~m:type:~x:.... when calling another bantype
@@ -199,24 +203,20 @@ int msgbypass_extban_is_ok(Client *client, Channel* channel, char *para_in, int
 	type = para;
 	matchby = strchr(para, ':');
 	if (!matchby || !matchby[1])
-		return msgbypass_extban_syntax(client, checkt, "Invalid syntax");
+		return msgbypass_extban_syntax(b->client, b->is_ok_check, "Invalid syntax");
 	*matchby++ = '\0';
 
 	if (!msgbypass_extban_type_ok(type))
-		return msgbypass_extban_syntax(client, checkt, "Unknown type");
+		return msgbypass_extban_syntax(b->client, b->is_ok_check, "Unknown type");
 
-	/* This is quite silly, we have to create a fake extban here due to
-	 * the current API of extban_conv_param_nuh and extban_conv_param_nuh_or_extban
-	 * expecting the full banmask rather than the portion that actually matters.
-	 */
-	snprintf(tmpmask, sizeof(tmpmask), "~?:%s", matchby);
-	if (extban_is_ok_nuh_extban(client, channel, tmpmask, checkt, what, what2) == 0)
+	b->banstr = matchby;
+	if (extban_is_ok_nuh_extban(b) == 0)
 	{
 		/* This could be anything ranging from:
 		 * invalid n!u@h syntax, unknown (sub)extbantype,
 		 * disabled extban type in conf, too much recursion, etc.
 		 */
-		return msgbypass_extban_syntax(client, checkt, "Invalid matcher");
+		return msgbypass_extban_syntax(b->client, b->is_ok_check, "Invalid matcher");
 	}
 
 	return 1; /* OK */
